﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace VirtualizingPanelSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {

        public ObservableCollection<Person> Persons { get; private set; }

        private TimeSpan _duration;
        public TimeSpan Duration
        {
            get { return _duration; }
            set
            {
                _duration = value;
                RaisePropertyChanged();
            }
        }

        private void RaisePropertyChanged([CallerMemberName] string propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public MainWindow()
        {
            InitializeComponent();
            Persons = new ObservableCollection<Person>();

            scrollUnit.Items.Add(ScrollUnit.Item);
            scrollUnit.Items.Add(ScrollUnit.Pixel);
            scrollUnit.SelectedIndex = 0;

            isVirtualizing.Items.Add(true);
            isVirtualizing.Items.Add(false);
            isVirtualizing.SelectedIndex = 1;

            isVirtualizingWhenGrouping.Items.Add(true);
            isVirtualizingWhenGrouping.Items.Add(false);
            isVirtualizingWhenGrouping.SelectedIndex = 1;

            this.DataContext = this;
        }

        private async void Button_Click(object sender, RoutedEventArgs e)
        {
            var testdata = Enumerable.Range(0, 10000).Select(i => new Person { Name = i.ToString(), Age = i, Gender = i % 2 == 0 ? "Male" : "Female" });
            foreach (var item in testdata)
            {
                Persons.Add(item);
            }
            TakeDispatcherTime();
        }

        private void TakeDispatcherTime()
        {
            Stopwatch watch = new Stopwatch();
            watch.Start();


            Dispatcher.Invoke(new Action(() => { }),
                  DispatcherPriority.Loaded, null);

            watch.Stop();
            Duration = watch.Elapsed;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            Persons.Clear();
            TakeDispatcherTime();
            MessageBox.Show(string.Format("VirtualizingPanel.IsVirtualizing: {0}{3}VirtualizingPanel.IsVirtualizingWhenGrouping: {1}{3}VirtualizingPanel.ScrollUnit: {2}{3}",
                lstView.GetValue(VirtualizingPanel.IsVirtualizingProperty),
                lstView.GetValue(VirtualizingPanel.IsVirtualizingWhenGroupingProperty),
                lstView.GetValue(VirtualizingPanel.ScrollUnitProperty),
                Environment.NewLine));
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            var cvs = this.Resources["personsView"] as CollectionViewSource;
            if (cvs.GroupDescriptions.Count == 0)
            {
                cvs.GroupDescriptions.Add(new PropertyGroupDescription("Gender"));
            }
            else
            {
                cvs.GroupDescriptions.Clear();
            }

            TakeDispatcherTime();
        }
    }
}
